#include "TChain.h"
#include "WebSelector.h"

#include "WebHEPInterface.h"
#include "TROOT.h"
#include "TRint.h"
#include "TEnv.h"
#include "TProof.h"
#include "TDSet.h"
#include <iostream>
#include <string>
#include "TObjString.h"
#include <fstream>
#include <time.h>

using namespace std;

const char* getJson(TProof* proof)
{
  if(proof->GetOutputList()->GetEntries() == 1)
  {
    TObjString* str = dynamic_cast<TObjString*>(proof->GetOutputList()->At(0));
    return str->GetName();
  }
  else
  {
    cerr << "Error: proof ouput list != 1"<< 
	proof->GetOutputList()->GetEntries()<< endl;
    return "";
  }
}

template <class T>
void printRoot(T* collection)
{
  TIter next(collection);
  while (TObject *obj = next())
	cerr <<"  "<< obj->GetName() << endl;
}

const char* runWebHEP(const char* inJson)
{

  string dir = "/home/jhugon/WebHEP/";
  string inDir = string(dir).append("www/data/");

  TChain* tree = new TChain("tree");
  
  tree->AddFile(string(inDir).append("lhcttbar.root").c_str());
  WebSelector select;

  select.SetOption(inJson);
  select.Begin(NULL);
  select.SlaveBegin(NULL);
  select.Init(tree);

  int treeSize = tree->GetEntries();
  cerr << "treeSize: " << treeSize << endl;
  int nEntries = treeSize;
  int maxEvents = 100000;
  if (treeSize > maxEvents)
	treeSize = maxEvents;

  cerr << "nEvents: " << treeSize << endl;
  int iPassed=0;

  for(int i=0; i<treeSize;i++)
  {
        if(!select.Process(i))
		continue;
	iPassed++;
	
  }
  select.SlaveTerminate();
  select.Terminate();

  cerr << "nEventsPassed: " << iPassed << endl;
  int outputlength = select.GetOutputList()->GetEntries();
  if (outputlength != 1)
  {
    cerr << "Error: TSelector ouput list != 1    "<< 
	outputlength << endl;
    return "";
  }
  TObjString* json = (TObjString*) select.GetOutputList()->At(0);
  cerr << "runWebHEP output: " << json->GetName() << endl;

  delete tree;
  cerr << "runWebHEP done.\n";
  return json->GetName();
}

const char* runWebHEPProof(const char* inJson)
{
  int nEventsProcess = -1;

  //gROOT->ProcessLine(".2>&1");
  cerr << "Starting runWebHEP...\n";
  TString tmpDir("/tmp/WebHEPInterface");
  tmpDir += time(NULL);
  gEnv->SetValue("ProofLite.Sandbox",tmpDir);

  TProof* proof = TProof::Open("workers=2");
  string dir = "/home/jhugon/WebHEP/";
  string inDir = string(dir).append("www/data/");
  string libLocation = string(dir).append("lib/libWebHEPSelector.so");
  
  proof->SetBit(TProof::kUsingSessionGui);
  proof->Load(libLocation.c_str());
  proof->Print(); 

  TDSet* ttbar = new TDSet("ttbar");
  TDSet* Zll = new TDSet("Zll");
  TDSet* Wlnu = new TDSet("Wlnu");
  TDSet* qcd = new TDSet("qcd");

  ttbar->Add(string(inDir).append("lhcttbar.root").c_str());
  Zll->Add(string(inDir).append("lhcZll.root").c_str());
  Wlnu->Add(string(inDir).append("lhcWlnu.root").c_str());
  qcd->Add(string(inDir).append("lhcqcd.root").c_str());

  ttbar->Process("WebSelector",inJson,nEventsProcess);
  string ttbarJson = getJson(proof);
  Zll->Process("WebSelector",inJson,nEventsProcess);
  string ZllJson = getJson(proof);
  Wlnu->Process("WebSelector",inJson,nEventsProcess);
  string WlnuJson = getJson(proof);
  qcd->Process("WebSelector",inJson,nEventsProcess);
  string qcdJson = getJson(proof);

  //cout << "\n\n ttbar: \n" << ttbarJson << endl;
  //cout << "\n\n Zll: \n" << ZllJson << endl;
  //cout << "\n\n Wlnu: \n" << WlnuJson << endl;
  //cout << "\n\n qcd: \n" << qcdJson << endl;

  string outJson = "{ ";
  outJson.append("'ttbar' : ");
  outJson.append(ttbarJson);
  outJson.append(" , ");
  outJson.append("'Zll' : ");
  outJson.append(ZllJson);
  outJson.append(" , ");
  outJson.append("'Wlnu' : ");
  outJson.append(WlnuJson);
  outJson.append(" , ");
  outJson.append("'qcd' : ");
  outJson.append(qcdJson);
  outJson.append(" }");

  //cerr << "runWebHEP results:\n";
  //cerr << outJson.c_str() << endl;

  delete ttbar;
  delete Zll;
  delete Wlnu;
  delete qcd;
  proof->Disconnect();
  gROOT->CloseFiles();

  cerr << "Proofs:\n";
  printRoot(gROOT->GetListOfProofs());
  cerr << "files:\n";
  printRoot(gROOT->GetListOfFiles());
/*
  cerr << "Cleanups:\n";
  printRoot(gROOT->GetListOfCleanups());
  cerr << "Globals:\n";
  printRoot(gROOT->GetListOfGlobals());
*/

  cerr << "runWebHEP done.\n";
  return outJson.c_str();
}

const char* WebHEPInterfaceTest(const char* inJson)
{
  return "WebHEPInterface Works!";
}
