#define WebSelector_cxx
// The class definition in WebSelector.h has been generated automatically
// by the ROOT utility TTree::MakeSelector(). This class is derived
// from the ROOT class TSelector. For more information on the TSelector
// framework see $ROOTSYS/README/README.SELECTOR or the ROOT User Manual.

// The following methods are defined in this file:
//    Begin():        called every time a loop on the tree starts,
//                    a convenient place to create your histograms.
//    SlaveBegin():   called after Begin(), when on PROOF called only on the
//                    slave servers.
//    Process():      called for each event, in this function you decide what
//                    to read and fill your histograms.
//    SlaveTerminate: called at the end of the loop on the tree, when on PROOF
//                    called only on the slave servers.
//    Terminate():    called at the end of the loop on the tree,
//                    a convenient place to draw/fit your histograms.
//
// To use this file, try the following session on your Tree T:
//
// Root > T->Process("WebSelector.C")
// Root > T->Process("WebSelector.C","some options")
// Root > T->Process("WebSelector.C+")
//

#include "WebSelector.h"
#include "PhysicsObjectFunctions.h"
#include "YamlReader.h"
#include "YamlWriter.h"
#include <TH2.h>
#include <TStyle.h>
#include <iostream>
#include "TObjString.h"

   using namespace std;

void WebSelector::Begin(TTree * /*tree*/)
{
   // The Begin() function is called at the start of the query.
   // When running with PROOF Begin() is only called on the client.
   // The tree argument is deprecated (on PROOF 0 is passed).

  ptMu1=0;
  ptEl1=0;
  ptPh1=0;
  ptJet1=0;

  histMet=0;
  histHt=0;
  histSt=0;
  histM=0;
  histSmin=0;
  histMDiMu=0;
  histMDiEl=0;
  histMDiJet=0;
  histMDiPh=0;
}

void WebSelector::SlaveBegin(TTree * /*tree*/)
{
   // The SlaveBegin() function is called after the Begin() function.
   // When running with PROOF SlaveBegin() is called on each slave server.
   // The tree argument is deprecated (on PROOF 0 is passed).


  int nBins = 25;
  ptMu1= new TH1F("ptMu1","",nBins,0,200);
  ptEl1= new TH1F("ptEl1","",nBins,0,200);
  ptPh1= new TH1F("ptPh1","",nBins,0,200);
  ptJet1= new TH1F("ptJet1","",nBins,0,300);

  histMet= new TH1F("histMet","",nBins,0,500);
  histHt= new TH1F("histHt","",nBins,0,1000);
  histSt= new TH1F("histSt","",nBins,0,1000);
  histM= new TH1F("histM","",nBins,0,1000);
  histSmin= new TH1F("histSmin","",nBins,0,1000);
  histMDiMu= new TH1F("histMDiMu","",nBins,0,200);
  histMDiEl= new TH1F("histMDiEl","",nBins,0,200);
  histMDiJet= new TH1F("histMDiJet","",nBins,0,500);
  histMDiPh= new TH1F("histMDiPh","",nBins,0,200);

  fOutput->Add(ptMu1);
  fOutput->Add(ptEl1);
  fOutput->Add(ptPh1);
  fOutput->Add(ptJet1);

  fOutput->Add(histMet);
  fOutput->Add(histHt);
  fOutput->Add(histSt);
  fOutput->Add(histM);
  fOutput->Add(histSmin);
  fOutput->Add(histMDiEl);
  fOutput->Add(histMDiMu);
  fOutput->Add(histMDiJet);
  fOutput->Add(histMDiPh);

   //string option = GetOption();
   //config = Config(option.c_str());
   const char* option = GetOption();
   //cout << option << endl;
  YamlReader reader(option);

  config.elpt = reader.GetValueSubMap("objectSelection","elpt");
  config.eleta = reader.GetValueSubMap("objectSelection","eleta");
  config.elpt = reader.GetValueSubMap("objectSelection","elpt");
  config.eleta = reader.GetValueSubMap("objectSelection","eleta");
  config.phpt = reader.GetValueSubMap("objectSelection","phpt");
  config.pheta = reader.GetValueSubMap("objectSelection","pheta");
  config.jetpt = reader.GetValueSubMap("objectSelection","jetpt");
  config.jeteta = reader.GetValueSubMap("objectSelection","jeteta");
  config.bTagEff = reader.GetValueSubMap("objectSelection","bTagEff");
  config.bTagFakeRate = reader.GetValueSubMap("objectSelection","bTagFakeRate");
  config.nMu = reader.GetValueSubMapInt("cutSelection","nMu");
  config.nEl = reader.GetValueSubMapInt("cutSelection","nEl");
  config.nPh = reader.GetValueSubMapInt("cutSelection","nPh");
  config.nJet = reader.GetValueSubMapInt("cutSelection","nJet");
  config.met = reader.GetValueSubMap("cutSelection","met");
}

Bool_t WebSelector::Process(Long64_t entry)
{
   // The Process() function is called for each entry in the tree (or possibly
   // keyed object in the case of PROOF) to be processed. The entry argument
   // specifies which entry in the currently loaded tree is to be processed.
   // It can be passed to either WebSelector::GetEntry() or TBranch::GetEntry()
   // to read either all or the required parts of the data. When processing
   // keyed objects with PROOF, the object is already loaded and is available
   // via the fObject pointer.
   //
   // This function should contain the "body" of the analysis. It can contain
   // simple or elaborate selection criteria, run algorithms on the data
   // of the event and typically fill histograms.
   //
   // The processing can be stopped by calling Abort().
   //
   // Use fStatus to set the return value of TTree::Process().
   //
   // The return value is currently not used.

   //cout << "in Process" << endl;
   fChain->GetEntry(entry);
   //cout << "entry gotten" << endl;

    if (met < config.met)
      return false;

    unsigned nMuons = muons->GetEntries();
    unsigned nElectrons = electrons->GetEntries();
    unsigned nPhotons = photons->GetEntries();
    unsigned nJets = jets->GetEntries();

    int nMuonsPass = 0;
    int nElectronsPass = 0;
    int nPhotonsPass = 0;
    int nJetsPass = 0;

    for(unsigned i=0; i<nMuons; i++)
    {
        TParticle* tmpMuon = (TParticle*) muons->At(i);
	if(tmpMuon->Pt() >= config.elpt && fabs(tmpMuon->Eta()) <= config.eleta)
	{
	  nMuonsPass++;
          //double relIso = getIso(tmpMuon);
	}
    }
    for(unsigned i=0; i<nElectrons; i++)
    {
        TParticle* tmpEl = (TParticle*) electrons->At(i);
	if(tmpEl->Pt() >= config.elpt && fabs(tmpEl->Eta()) <= config.eleta)
	{
	  nElectronsPass++;
          //double relIso = getIso(tmpEl);
	}
    }
    for(unsigned i=0; i<nPhotons; i++)
    {
        TParticle* tmpPh = (TParticle*) photons->At(i);
	if(tmpPh->Pt() >= config.phpt && fabs(tmpPh->Eta()) <= config.pheta)
	{
	  nPhotonsPass++;
          //double relIso = getIso(tmpPh);
	}
    }
    for(unsigned i=0; i<nJets; i++)
    {
        TParticle* tmpJet = (TParticle*) jets->At(i);
	if(tmpJet->Pt() >= config.jetpt && fabs(tmpJet->Eta()) <= config.jeteta)
	{
	  nJetsPass++;
	}
    }

    if(nMuonsPass < config.nMu || nElectronsPass < config.nEl || 
		nPhotonsPass < config.nPh || nJetsPass < config.nJet)
	return false;

   //////////////////
   /// Now Filling //
   //////////////////

    nMuonsPass = 0;
    nElectronsPass = 0;
    nPhotonsPass = 0;
    nJetsPass = 0;

    histMet->Fill(met);

    double ht=0;
    double st=met;
    TLorentzVector visible;
    TLorentzVector dijet;
    TLorentzVector dimu;
    TLorentzVector diel;
    TLorentzVector diph;

    for(unsigned i=0; i<nMuons; i++)
    {
        TParticle* tmpMuon = (TParticle*) muons->At(i);
	if(tmpMuon->Pt() >= config.elpt && fabs(tmpMuon->Eta()) <= config.eleta)
	{
	  nMuonsPass++;
	  TLorentzVector tmpP(tmpMuon->Px(),tmpMuon->Py(),tmpMuon->Pz(),tmpMuon->Energy());
	  st += tmpP.Pt();
	  visible += tmpP;
	  if(nMuonsPass==1)
	  {
		ptMu1->Fill(tmpMuon->Pt());
		dimu += tmpP;
	  }
	  if(nMuonsPass==2)
	  {
		dimu += tmpP;
	  }
          //double relIso = getIso(tmpMuon);
	}
    }
    for(unsigned i=0; i<nElectrons; i++)
    {
        TParticle* tmpEl = (TParticle*) electrons->At(i);
	if(tmpEl->Pt() >= config.elpt && fabs(tmpEl->Eta()) <= config.eleta)
	{
	  nElectronsPass++;
	  TLorentzVector tmpP(tmpEl->Px(),tmpEl->Py(),tmpEl->Pz(),tmpEl->Energy());
	  st += tmpP.Pt();
	  visible += tmpP;
	  if(nElectronsPass==1)
	  {
		ptEl1->Fill(tmpEl->Pt());
		diel += tmpP;
	  }
	  if(nElectronsPass==2)
	  {
		diel += tmpP;
	  }
          //double relIso = getIso(tmpEl);
	}
    }
    for(unsigned i=0; i<nPhotons; i++)
    {
        TParticle* tmpPh = (TParticle*) photons->At(i);
	if(tmpPh->Pt() >= config.phpt && fabs(tmpPh->Eta()) <= config.pheta)
	{
	  nPhotonsPass++;
	  TLorentzVector tmpP(tmpPh->Px(),tmpPh->Py(),tmpPh->Pz(),tmpPh->Energy());
	  st += tmpP.Pt();
	  visible += tmpP;
	  if(nPhotonsPass==1)
	  {
		ptPh1->Fill(tmpPh->Pt());
		diph += tmpP;
	  }
	  if(nPhotonsPass==2)
	  {
		diph += tmpP;
	  }
          //double relIso = getIso(tmpPh);
	}
    }
    for(unsigned i=0; i<nJets; i++)
    {
        TParticle* tmpJet = (TParticle*) jets->At(i);
	if(tmpJet->Pt() >= config.jetpt && fabs(tmpJet->Eta()) <= config.jeteta)
	{
	  nJetsPass++;
	  TLorentzVector tmpP(tmpJet->Px(),tmpJet->Py(),tmpJet->Pz(),tmpJet->Energy());
	  ht += tmpP.Pt();
	  st += tmpP.Pt();
	  visible += tmpP;
	  if(nJetsPass==1)
	  {
		ptJet1->Fill(tmpJet->Pt());
		dijet += tmpP;
	  }
	  if(nJetsPass==2)
	  {
		dijet += tmpP;
	  }
	}
    }

   histHt->Fill(ht);
   histSt->Fill(st);
   histM->Fill(visible.M());
   histSmin->Fill(sqrt(pow(visible.M(),2)+pow(visible.Pt(),2))+met);
   if(nMuonsPass>=2)
	histMDiMu->Fill(dimu.M());
   if(nElectronsPass>=2)
	histMDiEl->Fill(diel.M());
   if(nPhotonsPass>=2)
	histMDiPh->Fill(diph.M());
   if(nJetsPass>=2)
	histMDiJet->Fill(dijet.M());

   return true;
}

void WebSelector::SlaveTerminate()
{
   // The SlaveTerminate() function is called after all entries or objects
   // have been processed. When running with PROOF SlaveTerminate() is called
   // on each slave server.

}

void WebSelector::Terminate()
{
   // The Terminate() function is the last function to be called during
   // a query. It always runs on the client, it can be used to present
   // the results graphically or save the results to file.

   YamlWriter writer;
   writer.writeHistograms(fOutput); //only does TH1F*'s
   //writer.prettyPrint();
   TObjString* json = new TObjString(writer.getJson());
   fOutput->Clear();
   fOutput->Add(json);
}
