
#include "YamlWriter.h"
#include <iostream>
#include <string>

using namespace std;

void
YamlWriter::writeHistogram(TH1F* hist)
{
  using namespace YAML;
  emitter.SetStringFormat(SingleQuoted);//SingleQuoted, DoubleQuoted, Literal
  emitter.SetMapFormat(Flow);
  emitter.SetSeqFormat(Flow);
  //Flow is inline, Block is indented style
  // also setIndent(unsigned) may be useful
  //emitter.SetFloatPrecision(4); //only works in newer versions
  //emitter.SetDoublePrecision(4); //only works in newer versions

  int nBins = hist->GetXaxis()->GetNbins();

  emitter << BeginMap;
    emitter << Key << hist->GetName() << Value;
    emitter << BeginMap;
      emitter << Key << "'x'" << Value;
      emitter << BeginSeq;
	for(int i=1;i<nBins+2;i++)
	{
	  emitter << hist->GetXaxis()->GetBinLowEdge(i);
	}
      emitter << EndSeq;
      emitter << Key << "'y'" << Value;
      emitter << BeginSeq;
	for(int i=1;i<nBins+1;i++)
	{
	  emitter << hist->GetBinContent(i);
	}
      emitter << EndSeq;
    emitter << EndMap; //name map
  emitter << EndMap; //base map
}

void
YamlWriter::writeHistograms(TList* list)
{
  using namespace YAML;
  emitter.SetStringFormat(DoubleQuoted);//SingleQuoted, DoubleQuoted, Literal
  emitter.SetMapFormat(Flow);
  emitter.SetSeqFormat(Flow);
  //Flow is inline, Block is indented style
  // also setIndent(unsigned) may be useful
  //emitter.SetFloatPrecision(4); //only works in newer versions
  //emitter.SetDoublePrecision(4); //only works in newer versions

    emitter << BeginMap;
   TIter next(list);
   while (TObject *obj = next())
   {
     if(TH1F* hist = dynamic_cast<TH1F*>(obj))
     {
      int nBins = hist->GetXaxis()->GetNbins();
      //emitter << Key <<"'"<< hist->GetName() << "'" << Value;
      emitter << Key << hist->GetName()  << Value;
       emitter << BeginMap;
        emitter << Key << "x" << Value;
        emitter << BeginSeq;
	 for(int i=1;i<nBins+2;i++)
	 {
	  emitter << hist->GetXaxis()->GetBinLowEdge(i);
	 }
        emitter << EndSeq;
        emitter << Key << "y" << Value;
        emitter << BeginSeq;
	 for(int i=1;i<nBins+1;i++)
	 {
	  emitter << hist->GetBinContent(i);
	 }
        emitter << EndSeq;
      emitter << EndMap; //name map
     }
   }
  emitter << EndMap; //base map
}

void
YamlWriter::prettyPrint()
{

  cout << emitter.c_str() << endl;

}

const char*
YamlWriter::getJson()
{
  return emitter.c_str();
}
